module write_extr_map_m

  implicit none

contains

  subroutine write_extr_map(extr_map, longitude, latitude)

    use netcdf, only: nf90_clobber, NF90_FLOAT, NF90_INT
    use netcdf95, only: nf95_close, nf95_put_var, nf95_create, nf95_def_dim, &
         nf95_def_var, nf95_put_att, nf95_enddef

    integer, intent(in):: extr_map(:, :) ! map of extrema
    real, intent(in), optional:: longitude(:) ! in degrees
    real, intent(in), optional:: latitude(:) ! in degrees

    ! Local:
    integer ncid, varid_lat, varid_lon, varid_extr_map, dimid_lat, dimid_lon

    !--------------------------------------------------------------------------

    call nf95_create("extr_map.nc", NF90_CLOBBER, ncid)

    call nf95_def_dim(ncid, "lat", size(extr_map, 2), dimid_lat)
    call nf95_def_dim(ncid, "lon", size(extr_map, 1), dimid_lon)

    if (present(latitude)) then
       call nf95_def_var(ncid, "lat", NF90_FLOAT, dimid_lat, varid_lat)
       call nf95_put_att(ncid, varid_lat, "standard_name", "latitude")
       call nf95_put_att(ncid, varid_lat, "units", "degrees_north")
    end if

    if (present(longitude)) then
       call nf95_def_var(ncid, "lon", NF90_FLOAT, dimid_lon, varid_lon)
       call nf95_put_att(ncid, varid_lon, "standard_name", "longitude")
       call nf95_put_att(ncid, varid_lon, "units", "degrees_east")
    end if

    call nf95_def_var(ncid, "extr_map", NF90_INT, [dimid_lon, dimid_lat], &
         varid_extr_map)

    call nf95_enddef(ncid)

    if (present(longitude)) call nf95_put_var(ncid, varid_lon, longitude)
    if (present(latitude)) call nf95_put_var(ncid, varid_lat, latitude)
    call nf95_put_var(ncid, varid_extr_map, extr_map)

    call nf95_close(ncid)    
    print *, 'Created file "extr_map.nc".'

  end subroutine write_extr_map

end module write_extr_map_m
