#!/usr/bin/env python3

"""This script reads the eddy graph from Matlab v7.3 mat files and
    writes it in edgelist format. It is about 10 times slower than the
    corresponding script reading the same data from a Matlab v6 file.

"""

import h5py
import datetime
import csv
import sys

def convert_id(cyclone, eddy_index_Matlab):
    """Convert from Matlab eddy identification to eddy
    identification in shapefiles.

    We assume that the shapefiles are created by the script
    "mat_to_shapefiles.py", that is: at each date, all the
    anticyclones are before all the cyclones. We use global variable
    Nanti.

    """

    k = (eddy_index_Matlab - 1) // n_max
    date_num = Assoc_eddies["date_num"][k].item()
    date_num = int(date_num)
    date = datetime.date.fromordinal(date_num - 366) - datetime.date(1950,1,1)
    date = date.days
    
    if cyclone:
        eddy_index_F = Nanti[k] + eddy_index_Matlab - k * n_max
    else:
        eddy_index_F = eddy_index_Matlab - k * n_max

    return date, eddy_index_F

def print_edgelist(file, id_child, cyclone):
    for i1, id_child_1 in enumerate(id_child):
        if file[id_child_1].ndim == 2:
            assert file[id_child_1].shape[1] == 1
            predecessor = (cyclone, k1 * n_max + 1 + i1)
            # file[id_child_1][:, 0] is a numpy array of successors.

            for eddy_index_Matlab in file[id_child_1][:, 0]:
                eddy_index_Matlab = int(eddy_index_Matlab)
                writer.writerow(convert_id(*predecessor)
                                + convert_id(cyclone, eddy_index_Matlab))

orientation = sys.argv[1]
cyclone = orientation == "Cyclo"
Assoc_eddies = h5py.File(f"Association_eddies_{orientation}_max.mat", "r")

with h5py.File("Parameters_Anticyclonic_Eddies.mat", "r") as f:
    Nanti = f["Nanti"][:].squeeze().astype(int)
    
if cyclone:
    with h5py.File("Parameters_Cyclonic_Eddies.mat", "r") as f:
        n_eddies = f["Ncyclo"][:].squeeze().astype(int)
else:
    n_eddies = Nanti

n_dates = Nanti.size
print("n_dates =", n_dates)
n_max = Assoc_eddies["id_child"].shape[1]
print("n_max =", n_max)

with open(f"edgelist_{orientation}.csv", "w", newline = '') as edgelist:
    writer = csv.writer(edgelist, delimiter = ' ', lineterminator = "\n")
    writer.writerow(["predecessor date subscript", "predecessor eddy subscript",
                     "successor date subscript", "successor eddy subscript"])
    writer.writerow(["k1", "i1", "k2", "i2"])

    for k1 in range(n_dates):
        print("k1 =", k1)
        print_edgelist(Assoc_eddies,
                       Assoc_eddies["id_child"][k1, :n_eddies[k1]], cyclone)
