#!/usr/bin/env python3

"""This script reads the eddy graph from Matlab v6 files
id_child_*.mat and N_eddies.mat, for a given orientation, and writes
the graph to a new file in edgelist format. It also reads
date_num.mat.

"""

import scipy.io as sio
import csv
import datetime
import numpy as np

def to_days_1950(date_num):
    """Convert Matlab serial date number to number of days since Jan 1st
    1950.

    """
    date = datetime.date.fromordinal(date_num - 366) - datetime.date(1950, 1, 1)
    return date.days

def adjust_n(n_Matlab):
    k_Matlab = (n_Matlab - 1) // e_overestim
    k = date_num[k_Matlab] - date_num[0]
    return n_Matlab + (d_init + k - k_Matlab) * e_overestim

if __name__ == "__main__":
    n_files = input("Number of input files id_child_*.mat? ")
    n_files = int(n_files)
    n_eddies = sio.loadmat("N_eddies.mat", squeeze_me = True)["N_eddies"]\
                  .astype(int, casting = "safe", copy = False)
    date_num = sio.loadmat("date_num.mat", squeeze_me = True)["date_num"]
    id_child_list = []

    for i in range(n_files):
        print(f"Loading id_child_{i}.mat...")
        part_id_child = sio.loadmat(f"id_child_{i}.mat", squeeze_me = True)\
            ["part_id_child"]
        # part_id_child is a numpy array with shape (e_overestim, chunk)
        # where chunk is lower than or equal to the number of dates in the
        # original Matlab files.

        id_child_list.append(part_id_child)

    if n_files >= 2:
        id_child = np.concatenate(id_child_list, axis = 1)
        del id_child_list, part_id_child # can be a lot of memory
    else:
        id_child = part_id_child

    n_dates = n_eddies.size
    print("n_dates =", n_dates)
    e_overestim = id_child.shape[0]
    print("e_overestim =", e_overestim)
    d_init = to_days_1950(date_num[0])
    with open("e_overestim.txt", "w") as f: f.write(str(e_overestim) + "\n")

    with open("edgelist.csv", "w", newline = '') as edgelist:
        writer = csv.writer(edgelist, delimiter = ' ', lineterminator = "\n")

        for j1 in range(n_dates):
            for i1, id_child_1 in enumerate(id_child[:n_eddies[j1], j1]):
                n1 = (d_init + date_num[j1] - date_num[0]) * e_overestim + i1 \
                    + 1
                # id_child_1 may be an int, which means there is a single
                # successor, or id_child_1 may be a numpy array, which means
                # there are several successors.

                try:
                    # Try processing id_child_1 as an array:
                    for n_Matlab in id_child_1:
                        n2 = adjust_n(n_Matlab)
                        writer.writerow((n1, n2))
                except TypeError:
                    # id_child_1 is a scalar int.
                    n2 = adjust_n(id_child_1)
                    writer.writerow((n1, n2))
