#!/usr/bin/env python3

import sys

import cartopy.crs as ccrs
import matplotlib.pyplot as plt

import plot_eddy_contours
import util_eddies

# Input:
if len(sys.argv) != 6:
    sys.exit(
        "Required arguments: {Anticyclones,Cyclones} SHPC_dir e_overestim "
        "head-node tail-node"
    )
orientation = sys.argv[1]
SHPC_dir = sys.argv[2]
e_overestim = int(sys.argv[3])
my_head = int(sys.argv[4])
my_tail = int(sys.argv[5])

SHPC = util_eddies.SHPC_class(SHPC_dir, def_orient=orientation)
date_head, eddy_i_head = util_eddies.node_to_date_eddy(my_head, e_overestim)
date_tail, eddy_i_tail = util_eddies.node_to_date_eddy(my_tail, e_overestim)
print(f"{date_head=}")
print(f"{date_tail=}")
i_slice_head, ishape_head = SHPC.comp_ishape(
    date_head, eddy_i_head, orientation
)
i_slice_tail, ishape_tail = SHPC.comp_ishape(
    date_tail, eddy_i_tail, orientation
)
fig = plt.figure()
projection = ccrs.PlateCarree()
ax = plt.axes(projection=projection)
plot_eddy_contours.snapshot(ax, [ishape_head], SHPC, i_slice_head, orientation)
plot_eddy_contours.snapshot(
    ax, [ishape_tail], SHPC, i_slice_tail, orientation, dashed=True
)
ax.gridlines(draw_labels=True)
ax.coastlines()
plt.show()
