module config_graph_m

  implicit none

  integer, protected:: copy

  integer, protected:: max_delta = 1
  ! maximum interval of date indices between which we look for
  ! overlapping of eddies

  integer, protected:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the extremum of a given eddy.

  logical, protected:: cyclone = .true.

  real, protected:: min_inters_speed = 0.
  ! minimum intersection between maximum-speed contours, computed as a
  ! fraction of surface

  real, protected:: min_inters_out = 0.5
  ! minimum intersection between outermost contours, computed as a
  ! fraction of surface

contains

  subroutine config_graph(rank)

    ! Libraries:
#ifndef CPP_SEQUENTIAL
    use ezmpi, only: ezmpi_bcast
#endif
    use jumble, only: assert

    use read_grid_m, only: periodic, nlon

    integer, intent(in):: rank

    ! Local:
    namelist /config_graph_nml/ max_delta, dist_lim, cyclone, &
         min_inters_speed, min_inters_out

    !-----------------------------------------------------------------

    if (rank == 0) then
       ! config_graph_nml:
       write(unit = *, nml = config_graph_nml)
       print *, "Enter namelist config_graph_nml."
       read(unit = *, nml = config_graph_nml)
       write(unit = *, nml = config_graph_nml)
       call assert(max_delta >= 1, "config_graph max_delta")
       if (periodic) call assert(4 * dist_lim < nlon, "config_graph dist_lim")
       copy = merge(dist_lim, 0, periodic)
    end if

#ifndef CPP_SEQUENTIAL
    call ezmpi_bcast(max_delta, root = 0)
    call ezmpi_bcast(dist_lim, root = 0)
    call ezmpi_bcast(cyclone, root = 0)
    call ezmpi_bcast(min_inters_speed, root = 0)
    call ezmpi_bcast(min_inters_out, root = 0)
    call ezmpi_bcast(copy, root = 0)
#endif

  end subroutine config_graph

end module config_graph_m
