module inters_lines_m

  implicit none

contains

  logical function inters_lines(polyline_1, polyline_2, min_inters)

    ! Do the two polylines intersect with a sufficient area of
    ! intersection?

    ! Libraries:
    use contour_531, only: polyline
    use gpc_f, only: gpc_polygon_clip_f, GPC_INT, polygon

    use spher_polygon_area_m, only: spher_polygon_area
    use spher_polyline_area_m, only: spher_polyline_area

    type(polyline), intent(in):: polyline_1, polyline_2

    real, intent(in):: min_inters
    ! minimum intersection between polylines, computed as a fraction
    ! of surface

    ! Local:
    type(polygon) res_pol

    !-------------------------------------------------------------------------

    if (min_inters <= 1.) then
       call gpc_polygon_clip_f(GPC_INT, polygon(nparts = 1, &
            part = [polyline_1], hole = [.false.]), polygon(nparts = 1, &
            part = [polyline_2], hole = [.false.]), res_pol)

       if (res_pol%nparts == 0) then
          ! polyline_1 and polyline_2 do not overlap
          inters_lines = .false.
       else
          ! polyline_1 and polyline_2 overlap
          if (min_inters > 0.) then
             inters_lines = spher_polygon_area(res_pol) >= min_inters &
                  * min(abs(spher_polyline_area(polyline_1)), &
                  abs(spher_polyline_area(polyline_2)))
          else
             inters_lines = .true.
          end if
       end if
    else
       inters_lines = .false.
    end if

  end function inters_lines

end module inters_lines_m
