module read_eddy_m

  implicit none

contains

  subroutine read_eddy(e, d, i, hshp, ishape)

    use, intrinsic:: ISO_C_BINDING
    use, intrinsic:: ieee_arithmetic, only: ieee_value, IEEE_QUIET_NAN

    ! Libraries:
    use contour_531, only: null_polyline
    use gpc_f, only: shp_read_pol, polygon
    use jumble, only: deg_to_rad, pi
    use shapelib_03, only: dbf_read_attribute_03, shp_read_point

    use derived_types, only: eddy, shpc_slice_handler, missing_speed

    type(eddy), intent(out):: e
    ! All components except e%innermost_level are defined. Component
    ! e%extr%coord_proj may get its default initialization
    ! value. Components delta_in and delta_out get default
    ! initialization values.

    integer, intent(out):: d ! date
    integer, intent(out):: i ! eddy index
    TYPE(shpc_slice_handler), intent(in):: hshp
    integer, intent(in):: ishape ! 0-based

    ! Local:
    type(polygon) p
    real attr, point(2)

    !---------------------------------------------------------------------

    e%cyclone = hshp%cyclone

    ! extremum:
    call dbf_read_attribute_03(e%extr%ssh, hshp%extremum, hshp%extr_ssh, &
         ishape)
    call dbf_read_attribute_03(d, hshp%extremum, hshp%extr_date, ishape)
    call dbf_read_attribute_03(i, hshp%extremum, hshp%extr_eddy_index, ishape)
    call dbf_read_attribute_03(e%max_speed, hshp%extremum, hshp%extr_speed, &
         ishape)
    if (e%max_speed == missing_speed) &
         e%max_speed = ieee_value(0., IEEE_QUIET_NAN)
    call shp_read_point(e%extr%coord, hshp%extremum, ishape)
    e%extr%coord = e%extr%coord * deg_to_rad

    ! extr_proj:
    if (hshp%with_proj) then
       call shp_read_point(point, hshp%extr_proj, ishape)
       e%extr%coord_proj = nint(point)
    end if

    ! outermost_contour:

    call dbf_read_attribute_03(attr, hshp%outermost, hshp%out_r_eq_area, &
         ishape)
    e%out_cont%area = merge(pi * attr**2 * 1e6, - 1e6, attr >= 0.)
    ! * 1e6: km2 to m2

    call dbf_read_attribute_03(e%out_cont%ssh, hshp%outermost, hshp%out_ssh, &
         ishape)
    call shp_read_pol(p, hshp%outermost, ishape)

    if (p%nparts == 0) then
       e%out_cont%polyline = null_polyline()
    else
       e%out_cont%polyline = p%part(1)
       e%out_cont%polyline%points = e%out_cont%polyline%points * deg_to_rad
    end if

    ! max_speed_contour:

    call dbf_read_attribute_03(attr, hshp%max_speed, &
         hshp%max_speed_r_eq_area, ishape)
    e%speed_cont%area = merge(pi * attr**2 * 1e6, - 1e6, attr >= 0.)
    ! * 1e6: km2 to m2

    call dbf_read_attribute_03(e%speed_cont%ssh, hshp%max_speed, &
         hshp%max_speed_ssh, ishape)
    call shp_read_pol(p, hshp%max_speed, ishape)

    if (p%nparts == 0) then
       e%speed_cont%polyline = null_polyline()
    else
       e%speed_cont%polyline = p%part(1)
       e%speed_cont%polyline%points = e%speed_cont%polyline%points * deg_to_rad
    end if

  end subroutine read_eddy

end module read_eddy_m
