module read_grid_m

  implicit none

  integer, protected:: nlon = - 1, nlat = - 1
  ! size of ssh array in input NetCDF file, assuming no repeated point
  ! if the grid is global

  logical, protected:: periodic = .false. ! grid is periodic in longitude

  real, protected:: corner_whole(2) = huge(0.)
  ! longitude and latitude of the corner of the whole grid in input
  ! NetCDF file, in rad

  real, protected:: step(2) = huge(0.) ! longitude and latitude steps, in rad

  logical, protected:: uniform_lon_lat = .true.
  ! grid is cartesian and uniform in longitude and latitude

contains

  subroutine read_grid(shpc_dir, rank)

    ! Libraries:
#ifndef CPP_SEQUENTIAL
    use ezmpi, only: ezmpi_bcast
#endif
    use jumble, only: new_unit, deg_to_rad

    character(len = *), intent(in):: shpc_dir
    integer, intent(in):: rank

    ! Local:

    integer unit

    real:: corner_deg(2) = [huge(0.), huge(0.)]
    ! longitude and latitude of the corner of the whole grid in input
    ! NetCDF file, in degrees

    real:: step_deg(2) = [huge(0.), huge(0.)]
    ! longitude and latitude steps, in degrees

    real:: stag_u(2) = 0., stag_v(2) = 0.
    ! Staggering for u and v. That is, position in projection space x, y of
    ! point corresponding to u(1, 1) or v(1, 1) respective to point
    ! corresponding to ssh(1, 1).

    namelist /grid_nml/ corner_deg, step_deg, nlon, nlat, periodic, &
         uniform_lon_lat, stag_u, stag_v

    !-----------------------------------------------------------------

    if (rank == 0) then
       call new_unit(unit)
       open(unit, file = shpc_dir // "/grid_nml.txt", status = "old", &
            action = "read", position = "rewind")
       read(unit, nml = grid_nml)
       close(unit)

       if (uniform_lon_lat) then
          corner_whole = corner_deg * deg_to_rad
          step = step_deg * deg_to_rad
       end if
    end if

#ifndef CPP_SEQUENTIAL
    call ezmpi_bcast(nlon, root = 0)
    call ezmpi_bcast(nlat, root = 0)
    call ezmpi_bcast(periodic, root = 0)
    call ezmpi_bcast(uniform_lon_lat, root = 0)

    if (uniform_lon_lat) then
       call ezmpi_bcast(corner_whole, root = 0)
       call ezmpi_bcast(step, root = 0)
    end if
#endif

  end subroutine read_grid

end module read_grid_m
