module read_snapshot_m

  implicit none

contains

  subroutine read_snapshot(s, hshpc, ssm, k, copy)

    ! Libraries:
    use jumble, only: assert
    use numer_rec_95, only: hunt

    use derived_types, only: snapshot, eddy, shpc_slice_handler, shpc_slice_meta
    use read_eddy_m, only: read_eddy
    use read_grid_m, only: nlon, nlat, corner_whole, step, uniform_lon_lat

    type(snapshot), intent(out):: s
    ! completely defined except s%list%innermost_level

    TYPE(shpc_slice_handler), intent(in):: hshpc(:) ! (n_slices)
    type(shpc_slice_meta), intent(in):: ssm(:) ! (n_slices)
    integer, intent(in):: k ! date index
    integer, intent(in):: copy

    ! Local:
    integer ishape, ishape_first
    integer:: i_slice = huge(0)
    integer date_read
    integer eddy_i ! eddy index
    type(eddy) e

    !---------------------------------------------------------------------

    if (size(hshpc) == 1) then
       i_slice = 1
    else
       call hunt([ssm%d0, huge(0)], k, i_slice)
    end if

    if (k == ssm(i_slice)%d0) then
       ishape_first = 0
    else
       ! {k > ssm(i_slice)%d0}
       ishape_first = ssm(i_slice)%ishape_last(k - 1) + 1
    end if

    s%number_extr = ssm(i_slice)%ishape_last(k) - ishape_first + 1
    allocate(s%list(s%number_extr))

    do ishape = ishape_first, ssm(i_slice)%ishape_last(k)
       call read_eddy(e, date_read, eddy_i, hshpc(i_slice), ishape)

       ! Check that all the eddies have the same date index:
       if (date_read /= k) then
          print *, "read_snapshot: bad date index"
          print *, "date_read = ", date_read
          print *, "k = ", k
          print *, "ishape = ", ishape
          print *, "eddy_i = ", eddy_i
          stop 1
       end if

       s%list(eddy_i) = e
    end do

    ! If coord_proj for all eddies were not read from a shapefile then
    ! we define them here:
    if (s%number_extr /= 0 .and. uniform_lon_lat) &
         forall (eddy_i = 1:s%number_extr) s%list(eddy_i)%extr%coord_proj &
         = nint((s%list(eddy_i)%extr%coord - corner_whole) / step + 1.)

    ! Define s%extr_map from s%list%extr%coord_proj:

    allocate(s%extr_map(1 - copy:nlon + copy, nlat))
    s%extr_map(1:nlon, :) = 0

    ! Since we are not sure that there is no duplicate tuple in
    ! s%list%extr%coord_proj, we must use a do construct and not a forall:
    do eddy_i = 1, s%number_extr
       s%extr_map(s%list(eddy_i)%extr%coord_proj(1), &
            s%list(eddy_i)%extr%coord_proj(2)) = eddy_i
    end do

    call assert(count(s%extr_map(1:nlon, :) /= 0) == s%number_extr, &
         "read_snapshot: one eddy per grid point")

    ! Extend in longitude:
    s%extr_map(1 - copy:0, :) = s%extr_map(nlon + 1 - copy:nlon, :)
    s%extr_map(nlon + 1:nlon + copy, :) = s%extr_map(1:copy, :)

  end subroutine read_snapshot

end module read_snapshot_m
