module recv_snapshot_m

  implicit none

  private recv_ssh_contour

contains

  subroutine recv_snapshot(s, source, tag)

    ! Libraries:
    use ezmpi, only: ezmpi_recv

    use config_graph_m, only: copy
    use derived_types, only: snapshot
    use read_grid_m, only: nlon, nlat

    type(snapshot), intent(out):: s
    integer, intent(in):: source ! rank of sending MPI process
    integer, intent(in):: tag ! date index for received snapshot

    ! Local:
    integer i

    !-----------------------------------------------------------

    call ezmpi_recv(s%number_extr, source, tag)
    allocate(s%list(s%number_extr))

    do i = 1, s%number_extr
       call ezmpi_recv(s%list(i)%extr%coord, source, tag)
       call ezmpi_recv(s%list(i)%extr%coord_proj, source, tag)
       call ezmpi_recv(s%list(i)%extr%ssh, source, tag)
       call ezmpi_recv(s%list(i)%cyclone, source, tag)
       call recv_ssh_contour(s%list(i)%out_cont, source, tag)
       call recv_ssh_contour(s%list(i)%speed_cont, source, tag)
       call ezmpi_recv(s%list(i)%max_speed, source, tag)
       call ezmpi_recv(s%list(i)%delta_in, source, tag)
       call ezmpi_recv(s%list(i)%delta_out, source, tag)
    end do

    allocate(s%extr_map(1 - copy:nlon + copy, nlat))
    call ezmpi_recv(s%extr_map, source, tag)

  end subroutine recv_snapshot

  !*******************************************************************

  subroutine recv_ssh_contour(cont, source, tag)

    use ezmpi, only: ezmpi_recv
    use derived_types, only: ssh_contour

    type(ssh_contour), intent(out):: cont
    integer, intent(in):: source, tag

    !------------------------------------------------------------------

    call ezmpi_recv(cont%n_points, source, tag)
    call ezmpi_recv(cont%closed, source, tag)
    allocate(cont%points(2, cont%n_points))
    call ezmpi_recv(cont%points, source, tag)
    call ezmpi_recv(cont%ssh, source, tag)
    call ezmpi_recv(cont%area, source, tag)

  end subroutine recv_ssh_contour

end module recv_snapshot_m
