# What is it?

This repository contains software to detect and follow oceanic
meso-scale eddies. The code implements ideas and methods described in
the following articles: [Chaigneau et al.,
2009](https://doi.org/10.1016/j.pocean.2009.07.012); [Chaigneau et
al., 2011](https://doi.org/10.1029/2011JC007134); [Pegliasco et al.,
2015](https://doi.org/10.1002/2015jc010950); [Laxenaire et al.,
2018](https://doi.org/10.1029/2018JC014270).

## Context

Ssalto : segment sol multimissions d'altimétrie, d'orbitographie et de
localisation précise. Duacs : data unification and altimeter
combination system. MADT : maps of absolute dynamic topography and
absolute geostrophic velocities. Cf. aussi le glossaire sur le site
[Aviso](http://www.aviso.altimetry.fr/en/home.html).

Données altimétriques AVISO. Un satellite revient tous les sept jours
au même endroit. AVISO distribue un champ sur une grille, quotidien,
au quart de degré : la hauteur par rapport à l'ellipsoïde (ADT). Au
format NetCDF. On cherche des structures de 50 à 200 km de
rayon. Vitesse de rotation de l'ordre de 1 m s$^{- 1}$, vitesse du
centre de l'ordre de 1 dm s$^{- 1}$.  Propriétés de l'eau différentes
à l'intérieur et à l'extérieur d'un tourbillon. \Eng{Agulhas rings}
créés au sud de l'Afrique et transportés sur des années vers
l'Amérique du sud.

Prendre les données globales \Eng{delayed time gridded absolute
  dynamic topography} (DT-MADT) et \Eng{delayed time gridded
  geostrophic velocities anomalies} (DT-MSLA-UV).

[CMEMS](http://marine.copernicus.eu) a pris le relais d'AVISO pour la
distribution des données.

La norme de la vitesse géostrophique et la composante orthoradiale
doivent être à peu près constantes sur une ligne de niveau de la
hauteur.

## Vue d'ensemble du traitement informatique

Les données sont des champs globaux de SSH, u, v, sur un maillage au
quart de degré, quotidiens, sur 29 ans (environ 10$^4$ dates). Le
traitement informatique fonctionne indépendamment de la résolution du
maillage et de la durée considérée mais la résolution et la durée
visées orientent les choix de programmation pour un temps d'exécution,
une consommation de mémoire vive et des tailles de fichiers produits
raisonnables. Les résultats doivent être un ensemble de tourbillons
détectés à chaque date et un suivi des tourbillons entre les
différentes dates.

Le traitement peut être décomposé en trois parties : détection des
tourbillons instantanés ; graphe des recouvrements ; identification
des trajectoires.

# Installation

Dependencies: graph-tool networkx pyshp f90nml cartopy pygifsicle h5py
 pynco pygraphviz

If your operating system is Ubuntu 22.04 or more recent, the
certificate used by Gitlab will not be accepted by your
machine. Therefore you need to deactivate SSL verification when
contacting Gitlab:

```
$ GIT_SSL_NO_VERIFY=true git clone --recurse-submodules \
   https://gitlab.in2p3.fr/ipsl/lmd/dpao/Detection_eddies.git
$ cd Detection_eddies
$ git config http.sslVerify false
```

# Repartition of Fortran files into directories

Since commit 3289c070.

- `Inst_eddies`: May only contain program units which make up the
program `inst_eddies`. If a program unit does not go into
`inst_eddies`, it should not be here. Not all program units going into
`inst_eddies` must be here: some could be in directory Common.

- `Inst_eddies/Tests`: Main programs of unit tests for procedures in
directory `Inst_eddies`. Plus procedures used only in these unit tests.

- `Overlap`: May only contain program units which make up the program
`eddy_graph`. If a program unit does not go into `eddy_graph`, it
should not be here. Not all program units going into `eddy_graph` must
be here: some could be in directory Common.

- `Overlap/Tests`: Main programs of unit tests for procedures in
directory `Overlap`. Plus procedures used only in these unit tests.

- Common: Procedures that are used in both programs `inst_eddies` and
`eddy_graph`.

- Common/Tests: Main programs of unit tests for procedures in
directory `Common`. Plus procedures used only in these unit
tests. Plus procedures used in unit tests of both `Inst_eddies` and
`Overlap` directories but not used in either program `inst_eddies` nor
`eddy_graph`.
